@@ -40,15 +40,22 @@ enum class LinkTimeConstant : int32_t;
 #define JSC_COMMON_BYTECODE_INTRINSIC_FUNCTIONS_EACH_NAME(macro) \
     macro(argument) \
     macro(argumentCount) \
+    macro(arrayPush) \
     macro(getByIdDirect) \
     macro(getByIdDirectPrivate) \
+    macro(getPrototypeOf) \
     macro(getPromiseInternalField) \
     macro(getGeneratorInternalField) \
     macro(getAsyncGeneratorInternalField) \
+    macro(getAbstractModuleRecordInternalField) \
     macro(getArrayIteratorInternalField) \
     macro(getStringIteratorInternalField) \
+    macro(getMapIteratorInternalField) \
+    macro(getSetIteratorInternalField) \
     macro(idWithProfile) \
     macro(isObject) \
+    macro(isCallable) \
+    macro(isConstructor) \
     macro(isJSArray) \
     macro(isProxyObject) \
     macro(isDerivedArray) \
@@ -60,12 +67,15 @@ enum class LinkTimeConstant : int32_t;
     macro(isSet) \
     macro(isStringIterator) \
     macro(isArrayIterator) \
+    macro(isMapIterator) \
+    macro(isSetIterator) \
     macro(isUndefinedOrNull) \
     macro(tailCallForwardArguments) \
     macro(throwTypeError) \
     macro(throwRangeError) \
     macro(throwOutOfMemoryError) \
     macro(tryGetById) \
+    macro(tryGetByIdWithWellKnownSymbol) \
     macro(putByIdDirect) \
     macro(putByIdDirectPrivate) \
     macro(putByValDirect) \
@@ -74,6 +84,8 @@ enum class LinkTimeConstant : int32_t;
     macro(putAsyncGeneratorInternalField) \
     macro(putArrayIteratorInternalField) \
     macro(putStringIteratorInternalField) \
+    macro(putMapIteratorInternalField) \
+    macro(putSetIteratorInternalField) \
     macro(toNumber) \
     macro(toString) \
     macro(toObject) \
@@ -81,7 +93,6 @@ enum class LinkTimeConstant : int32_t;
     macro(newPromise) \
     macro(createPromise) \
     macro(createArgumentsButterfly) \
-    macro(defineEnumerableWritableConfigurableDataProperty) \
 
 #define JSC_COMMON_BYTECODE_INTRINSIC_CONSTANTS_EACH_NAME(macro) \
     JSC_COMMON_BYTECODE_INTRINSIC_CONSTANTS_SIMPLE_EACH_NAME(macro) \
@@ -122,9 +133,12 @@ enum class LinkTimeConstant : int32_t;
     macro(GeneratorStateCompleted) \
     macro(GeneratorStateExecuting) \
     macro(arrayIteratorFieldIndex) \
-    macro(arrayIteratorFieldIsDone) \
     macro(arrayIteratorFieldIteratedObject) \
     macro(arrayIteratorFieldKind) \
+    macro(mapIteratorFieldMapBucket) \
+    macro(mapIteratorFieldKind) \
+    macro(setIteratorFieldSetBucket) \
+    macro(setIteratorFieldKind) \
     macro(stringIteratorFieldIndex) \
     macro(stringIteratorFieldIteratedString) \
     macro(asyncGeneratorFieldSuspendReason) \
@@ -138,6 +152,8 @@ enum class LinkTimeConstant : int32_t;
     macro(AsyncGeneratorSuspendReasonYield) \
     macro(AsyncGeneratorSuspendReasonAwait) \
     macro(AsyncGeneratorSuspendReasonNone) \
+    macro(useIntlDateTimeFormatDayPeriod) \
+    macro(abstractModuleRecordFieldState) \
 
 #define JSC_COMMON_BYTECODE_INTRINSIC_CONSTANTS_CUSTOM_EACH_NAME(macro) \
     macro(sentinelMapBucket) \
