@@ -44,6 +44,7 @@ public:
         : InlineBox(renderer)
         , m_includeLogicalLeftEdge(false)
         , m_includeLogicalRightEdge(false)
+        , m_hasHardLinebreak(false)
         , m_descendantsHaveSameLineHeightAndBaseline(true)
         , m_baselineType(AlphabeticBaseline)
         , m_hasAnnotationsBefore(false)
@@ -201,13 +202,14 @@ public:
     
     void removeChild(InlineBox* child);
 
-    RenderObject::SelectionState selectionState() override;
+    RenderObject::HighlightState selectionState() override;
 
     bool canAccommodateEllipsis(bool ltr, int blockEdge, int ellipsisWidth) const final;
     float placeEllipsisBox(bool ltr, float blockLeftEdge, float blockRightEdge, float ellipsisWidth, float &truncatedWidth, bool&) override;
 
     bool hasTextChildren() const { return m_hasTextChildren; }
     bool hasTextDescendants() const { return m_hasTextDescendants; }
+    bool hasHardLinebreak() const { return m_hasHardLinebreak; }
     void setHasTextChildren() { m_hasTextChildren = true; setHasTextDescendants(); }
     void setHasTextDescendants() { m_hasTextDescendants = true; }
     
@@ -312,6 +314,7 @@ private:
     unsigned m_includeLogicalRightEdge : 1;
     unsigned m_hasTextChildren : 1;
     unsigned m_hasTextDescendants : 1;
+    unsigned m_hasHardLinebreak : 1;
     unsigned m_descendantsHaveSameLineHeightAndBaseline : 1;
 
 protected:
