@@ -35,6 +35,7 @@
 #include "HTMLElement.h"
 #include "HTMLSlotElement.h"
 #include "SVGElement.h"
+#include "SelectorCheckerTestFunctions.h"
 #include "SelectorCompiler.h"
 #include "SelectorFilter.h"
 #include "ShadowRoot.h"
@@ -154,7 +155,7 @@ void ElementRuleCollector::collectMatchingRules(const MatchRequest& matchRequest
 
     if (element().isLink())
         collectMatchingRulesForList(matchRequest.ruleSet->linkPseudoClassRules(), matchRequest);
-    if (SelectorChecker::matchesFocusPseudoClass(element()))
+    if (matchesFocusPseudoClass(element()))
         collectMatchingRulesForList(matchRequest.ruleSet->focusPseudoClassRules(), matchRequest);
     collectMatchingRulesForList(matchRequest.ruleSet->tagRules(element().localName(), element().isHTMLElement() && element().document().isHTMLDocument()), matchRequest);
     collectMatchingRulesForList(matchRequest.ruleSet->universalRules(), matchRequest);
@@ -200,7 +201,7 @@ void ElementRuleCollector::transferMatchedRules(DeclarationOrigin declarationOri
         addMatchedProperties({
             &matchedRule.ruleData->styleRule().properties(),
             static_cast<uint16_t>(matchedRule.ruleData->linkMatchType()),
-            static_cast<uint16_t>(matchedRule.ruleData->propertyWhitelistType()),
+            static_cast<uint16_t>(matchedRule.ruleData->propertyAllowlistType()),
             matchedRule.styleScopeOrdinal
         }, declarationOrigin);
     }
@@ -335,7 +336,7 @@ void ElementRuleCollector::collectMatchingShadowPseudoElementRules(const MatchRe
     ASSERT(element().containingShadowRoot()->mode() == ShadowRootMode::UserAgent);
 
     auto& rules = *matchRequest.ruleSet;
-#if ENABLE(VIDEO_TRACK)
+#if ENABLE(VIDEO)
     // FXIME: WebVTT should not be done by styling UA shadow trees like this.
     if (element().isWebVTTElement())
         collectMatchingRulesForList(rules.cuePseudoRules(), matchRequest);
@@ -384,8 +385,6 @@ void ElementRuleCollector::matchUserRules()
 void ElementRuleCollector::matchUARules()
 {
     // First we match rules from the user agent sheet.
-    if (UserAgentStyle::simpleDefaultStyleSheet)
-        m_result.isCacheable = false;
     auto* userAgentStyleSheet = m_isPrintStyle
         ? UserAgentStyle::defaultPrintStyle : UserAgentStyle::defaultStyle;
     matchUARules(*userAgentStyleSheet);
@@ -456,12 +455,11 @@ inline bool ElementRuleCollector::ruleMatches(const RuleData& ruleData, unsigned
     if (compiledSelector.status == SelectorCompilationStatus::SimpleSelectorChecker) {
         compiledSelector.wasUsed();
 
-        auto selectorChecker = SelectorCompiler::ruleCollectorSimpleSelectorCheckerFunction(compiledSelector);
 #if !ASSERT_MSG_DISABLED
         unsigned ignoreSpecificity;
-        ASSERT_WITH_MESSAGE(!selectorChecker(&element(), &ignoreSpecificity) || m_pseudoElementRequest.pseudoId == PseudoId::None, "When matching pseudo elements, we should never compile a selector checker without context unless it cannot match anything.");
+        ASSERT_WITH_MESSAGE(!SelectorCompiler::ruleCollectorSimpleSelectorChecker(compiledSelector, &element(), &ignoreSpecificity) || m_pseudoElementRequest.pseudoId == PseudoId::None, "When matching pseudo elements, we should never compile a selector checker without context unless it cannot match anything.");
 #endif
-        bool selectorMatches = selectorChecker(&element(), &specificity);
+        bool selectorMatches = SelectorCompiler::ruleCollectorSimpleSelectorChecker(compiledSelector, &element(), &specificity);
 
         if (selectorMatches && ruleData.containsUncommonAttributeSelector())
             m_didMatchUncommonAttributeSelector = true;
@@ -481,9 +479,7 @@ inline bool ElementRuleCollector::ruleMatches(const RuleData& ruleData, unsigned
 #if ENABLE(CSS_SELECTOR_JIT)
     if (compiledSelector.status == SelectorCompilationStatus::SelectorCheckerWithCheckingContext) {
         compiledSelector.wasUsed();
-
-        auto selectorChecker = SelectorCompiler::ruleCollectorSelectorCheckerFunctionWithCheckingContext(compiledSelector);
-        selectorMatches = selectorChecker(&element(), &context, &specificity);
+        selectorMatches = SelectorCompiler::ruleCollectorSelectorCheckerWithCheckingContext(compiledSelector, &element(), &context, &specificity);
     } else
 #endif // ENABLE(CSS_SELECTOR_JIT)
     {
@@ -495,7 +491,9 @@ inline bool ElementRuleCollector::ruleMatches(const RuleData& ruleData, unsigned
         }
         // Slow path.
         SelectorChecker selectorChecker(element().document());
-        selectorMatches = selectorChecker.match(*selector, element(), context, specificity);
+        selectorMatches = selectorChecker.match(*selector, element(), context);
+        if (selectorMatches)
+            specificity = selector->computeSpecificity();
     }
 
     if (ruleData.containsUncommonAttributeSelector()) {
